/* Using the code from my hermite/parabolic surface and the last programming lab, I'm trying
 * to make a hermite surface.  Yay.
 * Basically, this requires 4 parabolic equations.  Each one gives us the control points at
 * the desired value of t.  I am setting this up so that there are 5 windows:  4 have simple
 * hermite curves, and the 5th one displays the surface that they define.
 * 
 * I've tried to add support for rotating the solid with the mouse.  I'm sure there's a
 * relatively simple way to do this, since I've seen so many other people do it in projects,
 * but I can't really figure it out.
 * 
 * I have also (finally) started to optimize my kludgy curve code.  Originally, it had 3
 * steps:  Get the control points, calculate the geometry vectors, and then multiply them by
 * the basis matrix.  This was rather silly, as the "calculation" of geometry vectors just
 * meant copying them in a different order.  This wasted time and (since I saved all the info
 * at each step) space.  Now I am saving the control points in 2D arrays.  Read one way, they
 * have the coordinates for each control point, and the other way they are the geometry
 * vectors.  Convenient, since I need to think of it one way sometimes and the other way other
 * times.
 * 
 * Known issues:  Most importantly, I still don't have any good methods for doing the curves
 * in 3-space.  Right now, each of the control curves has a pre-set, unchangeable 
 * z-coordinate.  That kind of limits the power of the modelling program, though.
 * Right now the object is centered on the window based on the starting values of the curves.
 * Moving the control curves too much can result in clipping errors, as well as un-centering
 * it (which makes the rotating look bad).  These aren't significant problems.
 * Also, like I said above, the mouse control sucks.
 * 
 * Nevin, 12/11/99
 */

#include <stdlib.h>
#include <cmath>
//#include <time.h>
//#include <iostream.h>
#include <GLUT/glut.h>
#include "hermite.h"


// Global variables
int xRot, yRot, zRot;									// For rotating the solid object
int list, windows[5];									// display list, indices to windows
int numDimensions;										// Are we set up for 2D or 3D?
int windowHeight[4];
curveEnum dragPoint;
solidHermite object;
int clickX, clickY;										// For dragging in the solid window

// Set up 3-dimensional stuff...depth buffering, etc.
// Commented out, because this stuff isn't necessary until I do solid objects and lighting.
/*inline void threeD()
{	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	numDimensions = 3;
}

// Set up stuff for 2-dimensional windows.
inline void twoD()
{	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glClear(GL_DEPTH_BUFFER_BIT);
	numDimensions = 2;
}*/

void display0(void)
{	glClear (GL_COLOR_BUFFER_BIT);
//	if(numDimensions == 3)
//		twoD();
	object.draw(0);
	glutSwapBuffers();
}

void display1(void)
{	glClear (GL_COLOR_BUFFER_BIT);
//	if(numDimensions == 3)
//		twoD();
	object.draw(1);
	glutSwapBuffers();
}

void display2(void)
{	glClear (GL_COLOR_BUFFER_BIT);
//	if(numDimensions == 3)
//		twoD();
	object.draw(2);
	glutSwapBuffers();
}

void display3(void)
{	glClear (GL_COLOR_BUFFER_BIT);
//	if(numDimensions == 3)
//		twoD();
	object.draw(3);
	glutSwapBuffers();
}

void displaySolid(void)
{	glClear (GL_COLOR_BUFFER_BIT);
//	if(numDimensions == 2)
//		threeD();
	glPushMatrix();
	glRotatef(xRot, 1.0, 0.0, 0.0);
	glRotatef(yRot, 0.0, 1.0, 0.0);
	glRotatef(zRot, 0.0, 0.0, 1.0);
	glTranslatef(-105.0, -75.0, 0.0);
	object.drawSolid();
	glPopMatrix();
	glutSwapBuffers();
}

void init (void) 
{	// basic setup
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT);    
	
/*	GLfloat P1stuff[4][4] = { {	20, 40, 30, 1 },	// P1 curve's P1 coordinate
						  	  { 20, 70, 30, 1 },	// P1 curve's R1 coordinate
						  	  { 70, 20, 30, 1 },	// P1 curve's R4 coordinate
						  	  { 70, 40, 30, 1 } };	// P1 curve's P4 coordinate
	GLfloat R1stuff[4][4] = { { 20, 30, 50, 1 },	// etc.
							  { 20, 60, 50, 1 },
							  { 70, 10, 50, 1 },
							  { 70, 30, 50, 1 } };
	GLfloat R4stuff[4][4] = { { 20, 50,-50, 1 },
							  { 20, 10,-50, 1 },
							  { 70, 10,-50, 1 },
							  { 70, 30,-50, 1 } };
	GLfloat P4stuff[4][4] = { { 20, 40,-30, 1 },
							  { 20, 20,-30, 1 },
							  { 70, 20,-30, 1 },
							  { 70, 40,-30, 1 } };*/
	GLfloat P1s[4][3] = { { 70,100, 30 }, 		// the P1 coords for the 4 curves
						  { 70,100,-30 },		// (first the P1 curve, then R1, R4, P4)
						  { 70, 85, 50 },
						  { 70, 85,-50 } };
	GLfloat R1s[4][3] = { { 70,145, 30 },
						  { 70, 70,-30 },
						  { 70,130, 50 },
						  { 70, 55,-50 } };
	GLfloat R4s[4][3] = { {145, 70, 30 },
		 				  {145, 70,-30 },
		 				  {145, 55, 50 },
		 				  {145, 55,-50 } };
	GLfloat P4s[4][3] = { {145,100, 30 },
						  {145,100,-30 },
						  {145, 85, 50 },
						  {145, 85,-50 } };
	// Initialize global vars
	xRot = yRot = zRot = 0;
	numDimensions = 2;
	dragPoint = NONE;
	for(int i = 0; i < 4; i++)
	{	object.set(i, P1, P1s[i]);
		object.set(i, R1, R1s[i]);
		object.set(i, R4, R4s[i]);
		object.set(i, P4, P4s[i]);
	}
	
	
/*	list = glGenLists(1);		// See comment before defineList().
	
	// Set up initial display list
	defineList();	*/		
}

/*void reshape (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho(-150.0,150.0,-150.0,150.0,-150.0,150.0);
	glMatrixMode (GL_MODELVIEW);
}*/

void reshape0 (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	windowHeight[0] = h;
	glOrtho(0, w, 0, h, -29.0, -31.0);		// I hate the way this works...this is so that
											// the range will be +29 to +31.
}

void reshape1 (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	windowHeight[1] = h;
	glOrtho(0, w, 0, h, 31.0, 29.0);
}

void reshape2 (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	windowHeight[2] = h;
	glOrtho(0, w, 0, h, -49.0, -51.0);
}

void reshape3 (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	windowHeight[3] = h;
	glOrtho(0, w, 0, h, 51.0, 49.0);
}

void reshapeSolid (int w, int h)
{	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho(-150, 150, -150, 150, -150.0, 150.0);
}

void keyboard(unsigned char key, int x, int y)
{	switch (key) 
	{	case 27:
			exit(0);
			break;
		case 'x':
			xRot = (xRot + 10)%360;
			break;
		case 'X':
			xRot = (xRot - 10)%360;
			break;
		case 'y':
			yRot = (yRot + 10)%360;
			break;
		case 'Y':
			yRot = (yRot - 10)%360;
			break;
		case 'z':
			zRot = (zRot + 10)%360;
			break;
		case 'Z':
			zRot = (zRot - 10)%360;
			break;
   }
   int top = glutGetWindow();
   glutSetWindow(windows[4]);
   glutPostRedisplay();
   glutSetWindow(top);
}

void click0(int button, int state, int x, int y)
{	curveEnum choices[4] = {P1, R1, R4, P4};
	y = windowHeight[0] - y;		/* Convert from system's coordinate system to ours */
	dragPoint = NONE;
	if(state == GLUT_DOWN)
	{	int minDist = 10;
		for(int i = 0; i <= 3; i++)
			if(dist(x, y,object.get(0, choices[i], 0), object.get(0, choices[i], 1)) < minDist)
			{	minDist = dist(x, y, object.get(0, choices[i], 0), object.get(0, choices[i], 1));
				dragPoint = choices[i];
			}
	}
}

void click1(int button, int state, int x, int y)
{	curveEnum choices[4] = {P1, R1, R4, P4};
	y = windowHeight[1] - y;		/* Convert from system's coordinate system to ours */
	dragPoint = NONE;
	if(state == GLUT_DOWN)
	{	int minDist = 10;
		for(int i = 0; i <= 3; i++)
			if(dist(x, y,object.get(1, choices[i], 0), object.get(1, choices[i], 1)) < minDist)
			{	minDist = dist(x, y, object.get(1, choices[i], 0), object.get(1, choices[i], 1));
				dragPoint = choices[i];
			}
	}
}

void click2(int button, int state, int x, int y)
{	curveEnum choices[4] = {P1, R1, R4, P4};
	y = windowHeight[2] - y;		/* Convert from system's coordinate system to ours */
	dragPoint = NONE;
	if(state == GLUT_DOWN)
	{	int minDist = 10;
		for(int i = 0; i <= 3; i++)
			if(dist(x, y,object.get(2, choices[i], 0), object.get(2, choices[i], 1)) < minDist)
			{	minDist = dist(x, y, object.get(2, choices[i], 0), object.get(2, choices[i], 1));
				dragPoint = choices[i];
			}
	}
}

void click3(int button, int state, int x, int y)
{	curveEnum choices[4] = {P1, R1, R4, P4};
	y = windowHeight[3] - y;		/* Convert from system's coordinate system to ours */
	dragPoint = NONE;
	if(state == GLUT_DOWN)
	{	int minDist = 10;
		for(int i = 0; i <= 3; i++)
			if(dist(x, y,object.get(3, choices[i], 0), object.get(3, choices[i], 1)) < minDist)
			{	minDist = dist(x, y, object.get(3, choices[i], 0), object.get(3, choices[i], 1));
				dragPoint = choices[i];
			}
	}
	else dragPoint = NONE;
}

void clickSolid(int button, int state, int x, int y)
{	if(state == GLUT_DOWN)
	{	clickX = x;
		clickY = y;
	}
}

void drag0(int x, int y)
{	y = windowHeight[0] - y;		/* Convert from system's coordinate system to ours */
	if(dragPoint != NONE)
	{	object.set(0, dragPoint, 0, (GLfloat) x);
		object.set(0, dragPoint, 1, (GLfloat) y);
		glutPostRedisplay();
	}
	glutSetWindow(windows[4]);
	glutPostRedisplay();
	glutSetWindow(windows[0]);
}

void drag1(int x, int y)
{	y = windowHeight[1] - y;		/* Convert from system's coordinate system to ours */
	if(dragPoint != NONE)
	{	object.set(1, dragPoint, 0, (GLfloat) x);
		object.set(1, dragPoint, 1, (GLfloat) y);
		glutPostRedisplay();
	}
	glutSetWindow(windows[4]);
	glutPostRedisplay();
	glutSetWindow(windows[1]);
}

void drag2(int x, int y)
{	y = windowHeight[2] - y;		/* Convert from system's coordinate system to ours */
	if(dragPoint != NONE)
	{	object.set(2, dragPoint, 0, (GLfloat) x);
		object.set(2, dragPoint, 1, (GLfloat) y);
		glutPostRedisplay();
	}
	glutSetWindow(windows[4]);
	glutPostRedisplay();
	glutSetWindow(windows[2]);
}

void drag3(int x, int y)
{	y = windowHeight[3] - y;		/* Convert from system's coordinate system to ours */
	if(dragPoint != NONE)
	{	object.set(3, dragPoint, 0, (GLfloat) x);
		object.set(3, dragPoint, 1, (GLfloat) y);
		glutPostRedisplay();
	}
	glutSetWindow(windows[4]);
	glutPostRedisplay();
	glutSetWindow(windows[3]);
}

void dragSolid(int x, int y)
{	yRot = (yRot + 2 * (x - clickX)) % 360;
	xRot = (xRot + 2 * (y - clickY)) % 360;
	clickX = x;
	clickY = y;
	glutPostRedisplay();
}

int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);	// Is there any way to change
   																// this between windows?
   glutInitWindowSize (200, 200);
   glutInitWindowPosition (100, 100);

   
   windows[0] = glutCreateWindow ("P1 (z = 30)");
   glMatrixMode(GL_MODELVIEW);							// Moved from init() because I think
   glLoadIdentity();									// it has to be done for each window?
   glutDisplayFunc(display0);
   glutReshapeFunc(reshape0);
   glutMouseFunc(click0);
   glutMotionFunc(drag0);
   glutKeyboardFunc(keyboard);
   
   windows[1] = glutCreateWindow ("P4 (z = -50)");
   glutPositionWindow(110, 120);
   glMatrixMode(GL_MODELVIEW);							// Necessary each time?
   glLoadIdentity();
   glutDisplayFunc(display1);
   glutReshapeFunc(reshape1);
   glutMouseFunc(click1);
   glutMotionFunc(drag1);
   glutKeyboardFunc(keyboard);
   
   windows[2] = glutCreateWindow ("R1 (z = 50)");
   glutPositionWindow(120, 140);
   glMatrixMode(GL_MODELVIEW);							// Necessary each time?
   glLoadIdentity();
   glutDisplayFunc(display2);
   glutReshapeFunc(reshape2);
   glutMouseFunc(click2);
   glutMotionFunc(drag2);
   glutKeyboardFunc(keyboard);
   
   windows[3] = glutCreateWindow ("R4 (z = -30)");
   glutPositionWindow(130, 160);
   glMatrixMode(GL_MODELVIEW);							// Necessary each time?
   glLoadIdentity();
   glutDisplayFunc(display3);
   glutReshapeFunc(reshape3);
   glutMouseFunc(click3);
   glutMotionFunc(drag3);
   glutKeyboardFunc(keyboard);
   
   windows[4] = glutCreateWindow ("Solid Object");
   glutPositionWindow(350, 100);
   glutReshapeWindow(400, 400);
   glMatrixMode(GL_MODELVIEW);							// Necessary each time?
   glLoadIdentity();
   glutDisplayFunc(displaySolid);
   glutReshapeFunc(reshapeSolid);
   glutKeyboardFunc(keyboard);
   glutMouseFunc(clickSolid);
   glutMotionFunc(dragSolid);
   init ();
   
   glutMainLoop();
   return 0;  
}