#include "curve.h"

// Draws a cubic curve defined by parametric equations x(t), y(t), z(t).
// x(t) = at^3 + bt^2 + ct + d.  y(t) and z(t) have similar definitions with different
// coefficients.  The coefficients are in the x, y, and z arrays.  The curve will be a
// line strip made up of n lines.
void cubicCurve(GLfloat *x, GLfloat *y, GLfloat *z, GLint n)
{	GLfloat t = 0, t2 = 0, t3 = 0;
	GLfloat increment = 1.0 / n;
	
	glBegin(GL_LINE_STRIP);
		for(int i = 0; i <= n; i++)
		{	glVertex3f(	x[0] * t3 + x[1] * t2 + x[2] * t + x[3],
						y[0] * t3 + y[1] * t2 + y[2] * t + y[3],
						z[0] * t3 + z[1] * t2 + z[2] * t + z[3]);
			t += increment;
			t2 = t * t;
			t3 = t * t2;
		}
	glEnd();
}

// Returns the distance between (x1, y1) and (x2, y2).
float dist(int x1, int y1, int x2, int y2)
{	return sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
}
