#include "hermite.h"

solidHermite::solidHermite(): showPoints(true)
{	for(int i = 0; i < 4; i++)
	{	for(int j = 0; j < 3; j++)
			control[0][i][j] = control[1][i][j] = control[2][i][j] = control[3][i][j] = 0;
		calcCoeffs(i);
	}
	setDetail(30);
}

/*hermite::hermite(GLfloat *point1, GLfloat *point4, GLfloat *point7, GLfloat *control1, GLfloat *control4, GLfloat *control7)
: showPoints(true), detail(100), cont(C1)
{	for(int i = 0; i < 4; i++)
	{	p1[i] = point1[i];
		p4[i] = point4[i];
		p7[i] = point7[i];
		r1[i] = control1[i];
		r4a[i] = r4b[i] = control4[i];
		r7[i] = control7[i];
	}
	calcCoeffs();
}
	
hermite::hermite(GLfloat *point1, GLfloat *point4, GLfloat *point7, GLfloat *control1, GLfloat *control4a, GLfloat *control4b, GLfloat *control7)
: showPoints(true), detail(100), cont(C1)
{	for(int i = 0; i < 4; i++)
	{	p1[i] = point1[i];
		p4[i] = point4[i];
		p7[i] = point7[i];
		r1[i] = control1[i];
		r4a[i] = control4a[i];
		r4b[i] = control4b[i];
		r7[i] = control7[i];
	}
	calcCoeffs();
}*/

GLfloat solidHermite::get(int curveNum, curveEnum whichOne, int index)
{	if(curveNum < 0 || curveNum > 4)
	{	cerr << "Illegal curve number passed to solidHermite::get()\n";
		return 0;
	}
	switch(whichOne)
	{	case P1:		return control[curveNum][0][index];//return p1[curveNum][index];
		case P4:		return control[curveNum][1][index];//return p4[curveNum][index];
		case R1:		return control[curveNum][2][index];//return r1[curveNum][index];
		case R4:		return control[curveNum][3][index];//return r4[curveNum][index];
		case NONE:	return 0;
		default:		cerr << "Illegal enumerated type passed to solidHermite::get()\n.";
					return 0;
	}
}

void solidHermite::get(int curveNum, curveEnum whichOne, GLfloat *array)
{	if(curveNum < 0 || curveNum > 4)
	{	cerr << "Illegal curve number passed to solidHermite::get()\n";
		return;
	}
	switch(whichOne)
	{	case P1:		for(int i = 0; i < 3; i++)
						array[i] = control[curveNum][0][i];//p1[curveNum][i];
					break;
		case P4:		for(int i = 0; i < 3; i++)
						array[i] = control[curveNum][1][i];//p4[curveNum][i];
					break;
		case R1:		for(int i = 0; i < 3; i++)
						array[i] = control[curveNum][2][i];//r1[curveNum][i];
					break;
		case R4:		for(int i = 0; i < 3; i++)
						array[i] = control[curveNum][3][i];//r4[curveNum][i];
					break;
		case NONE:	break;
		default:		cerr << "Illegal enumerated type passed to solidHermite::get()\n.";
					break;
	}
}

int solidHermite::getDetail()
{	return detail;
}

bool solidHermite::displayingPoints()
{	return showPoints;
}

void solidHermite::set(int curveNum, curveEnum whichOne, int index, GLfloat value)
{	if(curveNum < 0 || curveNum > 4)
	{	cerr << "Illegal curve number passed to solidHermite::set()\n";
		return;
	}
	switch(whichOne)
	{	case P1:		control[curveNum][0][index] = value;//p1[curveNum][index] = value;
					break;
		case P4:		control[curveNum][1][index] = value;//p4[curveNum][index] = value;
					break;
		case R1:		control[curveNum][2][index] = value;//r1[curveNum][index] = value;
					break;
		case R4:		control[curveNum][3][index] = value;//r4[curveNum][index] = value;
					break;
		case NONE:	break;
		default:		cerr << "Illegal enumerated type passed to solidHermite::set().\n";
					break;
	}
	calcCoeffs(curveNum);
}

void solidHermite::set(int curveNum, curveEnum whichOne, GLfloat *array)
{	if(curveNum < 0 || curveNum > 4)
	{	cerr << "Illegal curve number passed to solidHermite::set()\n";
		return;
	}
	switch(whichOne)
	{	case P1:		for(int i = 0; i < 3; i++)
						control[curveNum][0][i] = array[i];//p1[curveNum][i] = array[i] ;
					break;
		case P4:		for(int i = 0; i < 3; i++)
						control[curveNum][1][i] = array[i];//p4[curveNum][i] = array[i];
					break;
		case R1:		for(int i = 0; i < 3; i++)
						control[curveNum][2][i] = array[i];//r1[curveNum][i] = array[i];
					break;
		case R4:		for(int i = 0; i < 3; i++)
						control[curveNum][3][i] = array[i];//r4[curveNum][i] = array[i];
					break;
		case NONE:	break;
		default:		cerr << "Illegal enumerated type passed to solidHermite::set().\n";
					break;
	}
	calcCoeffs(curveNum);
}

void solidHermite::setDetail(int arg)
{	if(arg > 0)
		detail = arg;
}

void solidHermite::displayPoints(bool arg)
{	showPoints = arg;
}

void solidHermite::draw(int curveNum)
{	if(curveNum < 0 || curveNum > 4)
		return;
	if(curveNum == 4)
	{	drawSolid();
		return;
	}
	if(showPoints)
	{	glPointSize(5.0);
		glBegin(GL_POINTS);
			glColor3f(0.3, 0.3, 0.5);		/* Tangent control points are blue-ish grey */
			glVertex3fv(control[curveNum][2]);//4fv(r1[curveNum]);
			glVertex3fv(control[curveNum][3]);//4fv(r4[curveNum]);
			glColor3f(1.0, 1.0, 1.0);		/* Endpoints are white */
			glVertex3fv(control[curveNum][0]);//4fv(p1[curveNum]);
			glVertex3fv(control[curveNum][1]);//4fv(p4[curveNum]);
		glEnd();
	}
	cubicCurve(xCoeffs[curveNum], yCoeffs[curveNum], zCoeffs[curveNum], detail);
}

void solidHermite::drawSolid()
{	static GLfloat M[4][4] = { { 2,-2, 1, 1 },			// Hermite basis matrix
					       {-3, 3,-2,-1 },
					       { 0, 0, 1, 0 },
					       { 1, 0, 0, 0 } };
	GLfloat * coords[3];
	GLfloat curP1[3], curR1[3], curR4[3], curP4[3], curGx[4], curGy[4], curGz[4], curXcoeffs[4], curYcoeffs[4], curZcoeffs[4];
	GLfloat t = 0, t2 = 0, t3 = 0, s = 0, s2 = 0, s3 = 0;
	GLfloat increment = 1.0 / detail;
	int offset = 0;		// 0 or detail+1.  Differentiates between starting points in coordinate array.
	int i;
	
	for(i = 0; i < 3; i++)
		coords[i] = new GLfloat [2 * (detail + 1)];
	
	while (t <= 1)
	{/*	curP1[0] = xCoeffs[0][0] * t3 + xCoeffs[0][1] * t2 + xCoeffs[0][2] * t + xCoeffs[0][3];
		curP1[1] = yCoeffs[0][0] * t3 + yCoeffs[0][1] * t2 + yCoeffs[0][2] * t + yCoeffs[0][3];
		curP1[2] = zCoeffs[0][0] * t3 + zCoeffs[0][1] * t2 + zCoeffs[0][2] * t + zCoeffs[0][3];
		curP4[0] = xCoeffs[1][0] * t3 + xCoeffs[1][1] * t2 + xCoeffs[1][2] * t + xCoeffs[1][3];
		curP4[1] = yCoeffs[1][0] * t3 + yCoeffs[1][1] * t2 + yCoeffs[1][2] * t + yCoeffs[1][3];
		curP4[2] = zCoeffs[1][0] * t3 + zCoeffs[1][1] * t2 + zCoeffs[1][2] * t + zCoeffs[1][3];
		curR1[0] = xCoeffs[2][0] * t3 + xCoeffs[2][1] * t2 + xCoeffs[2][2] * t + xCoeffs[2][3];
		curR1[1] = yCoeffs[2][0] * t3 + yCoeffs[2][1] * t2 + yCoeffs[2][2] * t + yCoeffs[2][3];
		curR1[2] = zCoeffs[2][0] * t3 + zCoeffs[2][1] * t2 + zCoeffs[2][2] * t + zCoeffs[2][3];
		curR4[0] = xCoeffs[3][0] * t3 + xCoeffs[2][1] * t2 + xCoeffs[2][2] * t + xCoeffs[2][3];
		curR4[1] = yCoeffs[3][0] * t3 + yCoeffs[2][1] * t2 + yCoeffs[2][2] * t + yCoeffs[2][3];
		curR4[2] = zCoeffs[3][0] * t3 + zCoeffs[2][1] * t2 + zCoeffs[2][2] * t + zCoeffs[2][3];
		
		curGx[0] = curP1[0];	curGx[1] = curP4[0]; curGx[2] = curR1[0];	curGx[3] = curR4[0];
		curGy[0] = curP1[1];	curGy[1] = curP4[1]; curGy[2] = curR1[1];	curGy[3] = curR4[1];
		curGz[0] = curP1[2];	curGz[1] = curP4[2]; curGz[2] = curR1[2];	curGz[3] = curR4[2];
	*/
		for(i = 0; i < 4; i++)
		{	curGx[i] = xCoeffs[i][0] * t3 + xCoeffs[i][1] * t2 + xCoeffs[i][2] * t + xCoeffs[i][3];
			curGy[i] = yCoeffs[i][0] * t3 + yCoeffs[i][1] * t2 + yCoeffs[i][2] * t + yCoeffs[i][3];
			curGz[i] = zCoeffs[i][0] * t3 + zCoeffs[i][1] * t2 + zCoeffs[i][2] * t + zCoeffs[i][3];
		}
	
		for(i = 0; i < 4; i++)
		{	curXcoeffs[i] = M[i][0] * curGx[0] + M[i][1] * curGx[1] + M[i][2] * curGx[2] + M[i][3] * curGx[3];
			curYcoeffs[i] = M[i][0] * curGy[0] + M[i][1] * curGy[1] + M[i][2] * curGy[2] + M[i][3] * curGy[3];
			curZcoeffs[i] = M[i][0] * curGz[0] + M[i][1] * curGz[1] + M[i][2] * curGz[2] + M[i][3] * curGz[3];
		}
		
		// Now draw the curve
		s = s2 = s3 = 0;
		glBegin(GL_LINE_STRIP);
			for(i = 0; i <= detail; i++)
			{	coords[0][i + offset] = curXcoeffs[0] * s3 + curXcoeffs[1] * s2 + curXcoeffs[2] * s + curXcoeffs[3];
				coords[1][i + offset] = curYcoeffs[0] * s3 + curYcoeffs[1] * s2 + curYcoeffs[2] * s + curYcoeffs[3];
				coords[2][i + offset] = curZcoeffs[0] * s3 + curZcoeffs[1] * s2 + curZcoeffs[2] * s + curZcoeffs[3];
				glVertex3f(coords[0][i+offset], coords[1][i+offset], coords[2][i+offset]);
				s += increment;
				s2 = s * s;
				s3 = s * s2;
			}
		glEnd();
		
		// Now draw the lines connecting this curve to the last one (if this isn't the first)
		if(t != 0)
		{	glBegin(GL_LINES);
				for(i = 0; i <= detail; i++)
				{	glVertex3f(coords[0][i], coords[1][i], coords[2][i]);
					glVertex3f(coords[0][i+detail+1], coords[1][i+detail+1], coords[2][i+detail+1]);
				}
			glEnd();
		}
		
		t += increment;
		t2 = t*t;
		t3 = t2 * t;
		offset = detail + 1 - offset;			// It's either 0 or (detail + 1).  Make it the other value.
	}
	for(i = 0; i < 3; i++)
		delete [] coords[i];
}

// Based on values of control points, this function calculates the coefficients to the parametric
// equation for the specified curve.
void solidHermite::calcCoeffs(int curveNum)
{	static GLfloat M[4][4] = { { 2,-2, 1, 1 },			// Hermite basis matrix
					       {-3, 3,-2,-1 },
					       { 0, 0, 1, 0 },
					       { 1, 0, 0, 0 } };
	//GLfloat Gx[4], Gy[4], Gz[4];					// Geometry vectors - not used anymore
	
	if(curveNum < 0 || curveNum > 4)
	{	cerr << "Illegal curve number passed to solidHermite::calcCoeffs().\n";
		return;
	}
	
/*	// Instead of doing this, which is basically unnecessary copying, we are now getting the
	// geometry vectors straight out of the control array.  See hermite.h for details.
	// Note that I am still doing the hermite calculations as if the R1 and R4 control points give the
	// tangent as calculated from the endpoints, rather than from the origin.  This makes the below
	// calculations ickier, but I will eventually change it back, because I think that it will give a better
	// visualization of how hermite tangents really are calculated.
	Gx[0] = p1[curveNum][0]; Gx[1] = p4[curveNum][0];
	Gx[2] = r1[curveNum][0] - p1[curveNum][0]; Gx[3] = r4[curveNum][0] - p4[curveNum][0];
	Gy[0] = p1[curveNum][1]; Gy[1] = p4[curveNum][1];
	Gy[2] = r1[curveNum][1] - p1[curveNum][1]; Gy[3] = r4[curveNum][1] - p4[curveNum][1];
	Gz[0] = p1[curveNum][2]; Gz[1] = p4[curveNum][2];
	Gz[2] = r1[curveNum][2] - p1[curveNum][2]; Gz[3] = r4[curveNum][2] - p4[curveNum][2];
*/	for(int i = 0; i < 4; i++)
	{	xCoeffs[curveNum][i] = M[i][0] * control[curveNum][0][0] + M[i][1] * control[curveNum][1][0] + M[i][2] * (control[curveNum][2][0] - control[curveNum][0][0]) + M[i][3] * (control[curveNum][3][0] - control[curveNum][1][0]);
		yCoeffs[curveNum][i] = M[i][0] * control[curveNum][0][1] + M[i][1] * control[curveNum][1][1] + M[i][2] * (control[curveNum][2][1] - control[curveNum][0][1]) + M[i][3] * (control[curveNum][3][1] - control[curveNum][1][1]);
		zCoeffs[curveNum][i] = M[i][0] * control[curveNum][0][2] + M[i][1] * control[curveNum][1][2] + M[i][2] * (control[curveNum][2][2] - control[curveNum][0][2]) + M[i][3] * (control[curveNum][3][2] - control[curveNum][1][2]);
//	{	xCoeffs[curveNum][i] = M[i][0] * Gx[0] + M[i][1] * Gx[1] + M[i][2] * Gx[2] + M[i][3] * Gx[3];
//		yCoeffs[curveNum][i] = M[i][0] * Gy[0] + M[i][1] * Gy[1] + M[i][2] * Gy[2] + M[i][3] * Gy[3];
//		zCoeffs[curveNum][i] = M[i][0] * Gz[0] + M[i][1] * Gz[1] + M[i][2] * Gz[2] + M[i][3] * Gz[3];
	}
}