/* This class contains significant changes to my previous curve code.  Most notably, instead of two
 * connected curves this has 4 unconnected curves which define a solid object.  The first one provides
 * the P1 coordinate at any given time t, the second provides P4, and the third and fourth are for R1 and
 * R4, respectively.  Nothing is done with continuity, as these curves do not directly relate to each other.
 * 
 * Also important is the new way the control points are stored.  I used to keep each point in arrays for
 * that point, and when I needed the geometry vectors, I would have to copy them into a new order.  Now,
 * I have all the points in a 3x4 (or is it 4x3?) matrix, so they can be read one way to get the coordinates
 * of the control points and the other way for the geometry vectors.  Actually, since there are 4 curves, I
 * have a 3-dimensional array:  [4][4][3].  The first subscript references the [4][3] matrix for the desired
 * curve.  In that 2-D array, you can get the coordinates for a control point by looking at the 3 values going
 * one way.  (For example, P1 of curve #i is at (control[i][0][0], control[i][0][1], control[i][0][2]).)  To get the
 * geometry vectors, simply read down.  The x values defining curve #i are control[i][0][0], control[i][1][0],
 * control[i][2][0], and control[i][3][0].
 */
 
#ifndef _NEVIN_S_HERMITE_
#define _NEVIN_S_HERMITE_

#include <iostream>
#include "curve.h"
using namespace std;

class solidHermite
{	public:
		solidHermite();
//		solidHermite(GLfloat*, GLfloat*, GLfloat*, GLfloat*, GLfloat*, GLfloat*, GLfloat*);
//		solidHermite(GLfloat*, GLfloat*, GLfloat*, GLfloat*, GLfloat*, GLfloat*);
		
		GLfloat get(int, curveEnum, int);
		void get(int, curveEnum, GLfloat*);
		int getDetail();
		bool displayingPoints();
		
		void set(int, curveEnum, int, GLfloat);
		void set(int, curveEnum, GLfloat*);
		void setDetail(int);
		void displayPoints(bool);
		
		void draw(int);
		void drawSolid();
				
	private:
		void calcCoeffs(int);
		void calcC(GLfloat *G, GLfloat *C);
		friend void cubicCurve(GLfloat*, GLfloat*, GLfloat*, GLint);
		
//		GLfloat p1[4][4], p4[4][4], r1[4][4], r4[4][4];					// control points
		GLfloat control[4][4][3];								// control points
		GLfloat M[4][4];										// basis matrix
		GLfloat xCoeffs[4][4], yCoeffs[4][4], zCoeffs[4][4];			// parametric coefficients
		bool showPoints;									// display the control pts?
		int detail;											// how many line segments
};

#endif